@@ -8,19 +8,19 @@
 
 BEGIN { $| = 1; print "1..1\n"; }
 END {print "not ok 1\n" unless $loaded;}
-use CGI::WPM::SequentialFile 1.04;
+use CGI::WPM::FileVirtualPath 0.35;
 use CGI::WPM::EventCountFile 1.04;
 use CGI::WPM::PageMaker 1.02;
 use CGI::WPM::WebUserIO 0.94;
-use CGI::WPM::Globals 0.34;
+use CGI::WPM::Globals 0.35;
 use CGI::WPM::Base 0.34;
 use CGI::WPM::MultiPage 0.34;
 use CGI::WPM::Static 0.34;
-use CGI::WPM::MailForm 0.34;
-use CGI::WPM::GuestBook 0.34;
+use CGI::WPM::MailForm 0.35;
+use CGI::WPM::GuestBook 0.35;
 use CGI::WPM::SegTextDoc 0.34;
 use CGI::WPM::Redirect 0.34;
-use CGI::WPM::Usage 0.34;
+use CGI::WPM::Usage 0.35;
 $loaded = 1;
 print "ok 1\n";
 use strict;
